<?php
/*
* 2007-2015 PrestaShop
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author PrestaShop SA <contact@prestashop.com>
*  @copyright  2007-2015 PrestaShop SA
*  @license    http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of PrestaShop SA
*/

use PrestaShop\PrestaShop\Core\Payment\PaymentOption;

if (!defined('_PS_VERSION_')) {
    exit;
}

class BzpayPayment extends PaymentModule
{
    private $_html = '';
    protected $_postErrors = array();

    public $details;
    public $owner;
    public $address;
    public $extra_mail_vars;

    public function __construct()
    {
        $this->name = 'bzpaypayment';
        $this->tab = 'payments_gateways';
        $this->version = '1.0.0';
        $this->ps_versions_compliancy = array('min' => '1.7', 'max' => _PS_VERSION_);
        $this->author = 'Bzpay Solutions';
        $this->controllers = array('validation');
        $this->is_eu_compatible = 1;

        $this->currencies = true;
        $this->currencies_mode = 'checkbox';

        $this->bootstrap = true;
        parent::__construct();

        $this->displayName = $this->l('BzpayPayment');
        $this->description = $this->l('Description of Payment');

        if (!count(Currency::checkPaymentCurrencies($this->id))) {
            $this->warning = $this->l('No currency has been set for this module.');
        }
    }

    public function install()
    {
		
		Configuration::updateValue( 'BZPAY_MERCHANT_ID', '' );
		Configuration::updateValue( 'BZPAY_AUTH_KEY', '' );
		Configuration::updateValue( 'BZPAY_MSI', '' );
		Configuration::updateValue( 'BZPAY_MSI_3' );
		Configuration::updateValue( 'BZPAY_MSI_6' );
		Configuration::updateValue( 'BZPAY_MSI_9' );
		Configuration::updateValue( 'BZPAY_MSI_12' );
		Configuration::updateValue( 'MSI_PROMO_3' );
		Configuration::updateValue( 'MSI_PROMO_6' );
		Configuration::updateValue( 'MSI_PROMO_9' );
		Configuration::updateValue( 'MSI_PROMO_12' );
		
		
		
		
		
		
				
		return ( parent::install()
		         && $this->registerHook( 'paymentOptions' )
		         && $this->registerHook( 'paymentReturn' )
				 && $this->installDb() 
		          );
		
        
        
    }
	
	public function installDb() {
		return (
			Db::getInstance()->Execute( 'CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . 'bzpay` (
                `id`				INT(11) NOT NULL AUTO_INCREMENT,
                `bzpay_tid`		VARCHAR(255) NOT NULL,
                `order_id`			INT(11) NOT NULL,
                `payed_at`			DATETIME NOT NULL,
                `payed_amount`		DECIMAL(20,6) NOT NULL,
                `refunded_amount`	DECIMAL(20,6) NOT NULL,
                `captured`		    VARCHAR(255) NOT NULL,
                `msi`		    VARCHAR(255) NOT NULL,
                `promo`		    VARCHAR(255) NOT NULL,
                PRIMARY KEY			(`id`)
                ) ENGINE=InnoDB		DEFAULT CHARSET=latin1 AUTO_INCREMENT=1 ;' )
		);
	}
	public function uninstall() {
	
		//Fetch all languages and delete Paylike configurations which has lang

		return (
			parent::uninstall()
			&& Db::getInstance()->Execute( 'DROP TABLE IF EXISTS `' . _DB_PREFIX_ . 'bzpay`' )
			&& Configuration::deleteByName( 'BZPAY_MERCHANT_ID' )
			&& Configuration::deleteByName( 'BZPAY_AUTH_KEY' )
			&& Configuration::deleteByName( 'BZPAY_MSI' )
			&& Configuration::deleteByName( 'BZPAY_MSI_3' )
			&& Configuration::deleteByName( 'BZPAY_MSI_6' )
			&& Configuration::deleteByName( 'BZPAY_MSI_9' )
			&& Configuration::deleteByName( 'BZPAY_MSI_12' )
			&& Configuration::deleteByName( 'MSI_PROMO_3' )
	    	&& Configuration::deleteByName( 'MSI_PROMO_6' )
	    	&& Configuration::deleteByName( 'MSI_PROMO_9' )
	    	&& Configuration::deleteByName( 'MSI_PROMO_12' )
		
			
			
			
			
		);
	}
	
	public function getContent() {
		$this->html = '';
		if ( Tools::isSubmit( 'submitBzPay' ) ) {
			$valid = true;
	
			$BZPAY_MERCHANT_ID = ( ! empty( Tools::getvalue( 'BZPAY_MERCHANT_ID' ) ) ) ? Tools::getvalue( 'BZPAY_MERCHANT_ID' ) : '';
			$BZPAY_AUTH_KEY = ( ! empty( Tools::getvalue( 'BZPAY_AUTH_KEY' ) ) ) ? Tools::getvalue( 'BZPAY_AUTH_KEY' ) : '';
			$BZPAY_MSI = ( ! empty( Tools::getvalue( 'BZPAY_MSI' ) ) ) ? Tools::getvalue( 'BZPAY_MSI' ) : '';
			
			$BZPAY_MSI_3 = ( ! empty( Tools::getvalue( 'BZPAY_MSI_3' ) ) ) ? Tools::getvalue( 'BZPAY_MSI_3' ) : '';
			$BZPAY_MSI_6 = ( ! empty( Tools::getvalue( 'BZPAY_MSI_6' ) ) ) ? Tools::getvalue( 'BZPAY_MSI_6' ) : '';
			$BZPAY_MSI_9 = ( ! empty( Tools::getvalue( 'BZPAY_MSI_9' ) ) ) ? Tools::getvalue( 'BZPAY_MSI_9' ) : '';
			$BZPAY_MSI_12 = ( ! empty( Tools::getvalue( 'BZPAY_MSI_12' ) ) ) ? Tools::getvalue( 'BZPAY_MSI_12' ) : '';
			
			$MSI_PROMO_3 = ( ! empty( Tools::getvalue( 'MSI_PROMO_3' ) ) ) ? Tools::getvalue( 'MSI_PROMO_3' ) : '';
			$MSI_PROMO_6 = ( ! empty( Tools::getvalue( 'MSI_PROMO_6' ) ) ) ? Tools::getvalue( 'MSI_PROMO_6' ) : '';
			$MSI_PROMO_9 = ( ! empty( Tools::getvalue( 'MSI_PROMO_9' ) ) ) ? Tools::getvalue( 'MSI_PROMO_9' ) : '';
			$MSI_PROMO_12 = ( ! empty( Tools::getvalue( 'MSI_PROMO_12' ) ) ) ? Tools::getvalue( 'MSI_PROMO_12' ) : '';
				
			
				Configuration::updateValue( 'BZPAY_MERCHANT_ID', $BZPAY_MERCHANT_ID );
				Configuration::updateValue( 'BZPAY_AUTH_KEY', $BZPAY_AUTH_KEY );
				Configuration::updateValue( 'BZPAY_MSI', $BZPAY_MSI );
				Configuration::updateValue( 'BZPAY_MSI_3', $BZPAY_MSI_3 );
				Configuration::updateValue( 'BZPAY_MSI_6', $BZPAY_MSI_6 );
				Configuration::updateValue( 'BZPAY_MSI_9', $BZPAY_MSI_9 );
				Configuration::updateValue( 'BZPAY_MSI_12', $BZPAY_MSI_12 );
				Configuration::updateValue( 'MSI_PROMO_3',$MSI_PROMO_3 );
		        Configuration::updateValue( 'MSI_PROMO_6',$MSI_PROMO_6 );
		        Configuration::updateValue( 'MSI_PROMO_9',$MSI_PROMO_9 );
		        Configuration::updateValue( 'MSI_PROMO_12',$MSI_PROMO_12 );
				
			

			if ( $valid ) {
				$this->context->controller->confirmations[] = $this->l( 'Settings saved successfully' );
			}
		}

		//Get configuration form
		$this->html .= $this->renderForm();

		//$this->html .= $this->getModalForAddMoreLogo();

		$this->context->controller->addJS( $this->_path . 'views/js/backoffice.js' );

		return $this->html;
	}

public function renderForm() {
		

		//Set configuration form fields
		$fields_form = array(
			'form' => array(
				'legend' => array(
					'title' => $this->l( 'BzPay Payments Settings' ),
					'icon'  => 'icon-cogs'
				),
				'input'  => array(
					array(
						'type'     => 'text',
						'label'    => '<span data-toggle="tooltip" title="' . $this->l( 'Get it from your BzPay advisor' ) . '">' . $this->l( 'BZPAY MERCHANT ID' ) . '<i class="process-icon-help-new help-icon" aria-hidden="true"></i></span>',
						'name'     => 'BZPAY_MERCHANT_ID',
						'class'    => 'paylike-config',
						'required' => true
					),
					array(
						'type'     => 'text',
						'label'    => '<span data-toggle="tooltip" title="' . $this->l( 'Get it from your BzPay advisor' ) . '">' . $this->l( 'BZPAY AUTH KEY' ) . '<i class="process-icon-help-new help-icon" aria-hidden="true"></i></span>',
						'name'     => 'BZPAY_AUTH_KEY',
						'class'    => 'paylike-config',
						'required' => true
					),
				    array(
	                    'type' => 'switch',
	                    'label'    => '<span data-toggle="tooltip" title="' . $this->l( 'Activar MSI' ) . '">' . $this->l( 'MSI' ) . '<i class="process-icon-help-new help-icon" aria-hidden="true"></i></span>',
	                    'name'     => 'BZPAY_MSI',
	                    'is_bool' => true,
	                    'class'    => 'paylike-config',
	                    'required' => true,
	                    'values' => array(
		                    array(
			                'id' => 'active_on',
			                'value' => 1,
			                'label' => $this->l('Yes')
		                    ),
		                        array(
			                    'id' => 'active_off',
			                    'value' => 0,
			                    'label' => $this->l('No')
			                    )
	                    ),
	                   ),
	                   array(
						'type'     => 'text',
						'label'    => '<span data-toggle="tooltip" title="' . $this->l( 'Get it from your BzPay advisor' ) . '">' . $this->l( 'MONTO PROMOCION TRES MSI' ) . '<i class="process-icon-help-new help-icon" aria-hidden="true"></i></span>',
						'name'     => 'BZPAY_MSI_3',
						'class'    => 'paylike-config',
						'required' => false
					),
					  array(
	                    'type' => 'switch',
	                    'label'    => '<span data-toggle="tooltip" title="' . $this->l( 'Activar PROMCION' ) . '">' . $this->l( 'PROMO3' ) . '<i class="process-icon-help-new help-icon" aria-hidden="true"></i></span>',
	                    'name'     => 'MSI_PROMO_3',
	                    'is_bool' => true,
	                    'class'    => 'paylike-config',
	                    'required' => true,
	                    'values' => array(
		                    array(
			                'id' => 'active_on',
			                'value' => 1,
			                'label' => $this->l('Yes')
		                    ),
		                        array(
			                    'id' => 'active_off',
			                    'value' => 0,
			                    'label' => $this->l('No')
			                    )
	                    ),
	                   ),
					    array(
						'type'     => 'text',
						'label'    => '<span data-toggle="tooltip" title="' . $this->l( 'Get it from your BzPay advisor' ) . '">' . $this->l( 'MONTO PROMOCION SEIS MSI' ) . '<i class="process-icon-help-new help-icon" aria-hidden="true"></i></span>',
						'name'     => 'BZPAY_MSI_6',
						'class'    => 'paylike-config',
						'required' => false
					),
					 array(
	                    'type' => 'switch',
	                    'label'    => '<span data-toggle="tooltip" title="' . $this->l( 'Activar PROMCION' ) . '">' . $this->l( 'PROMO6' ) . '<i class="process-icon-help-new help-icon" aria-hidden="true"></i></span>',
	                    'name'     => 'MSI_PROMO_6',
	                    'is_bool' => true,
	                    'class'    => 'paylike-config',
	                    'required' => true,
	                    'values' => array(
		                    array(
			                'id' => 'active_on',
			                'value' => 1,
			                'label' => $this->l('Yes')
		                    ),
		                        array(
			                    'id' => 'active_off',
			                    'value' => 0,
			                    'label' => $this->l('No')
			                    )
	                    ),
	                   ),
					    array(
						'type'     => 'text',
						'label'    => '<span data-toggle="tooltip" title="' . $this->l( 'Get it from your BzPay advisor' ) . '">' . $this->l( 'MONTO PROMOCION NUEVE MSI' ) . '<i class="process-icon-help-new help-icon" aria-hidden="true"></i></span>',
						'name'     => 'BZPAY_MSI_9',
						'class'    => 'paylike-config',
						'required' => false
					),
					 array(
	                    'type' => 'switch',
	                    'label'    => '<span data-toggle="tooltip" title="' . $this->l( 'Activar PROMCION' ) . '">' . $this->l( 'PROMO9' ) . '<i class="process-icon-help-new help-icon" aria-hidden="true"></i></span>',
	                    'name'     => 'MSI_PROMO_9',
	                    'is_bool' => true,
	                    'class'    => 'paylike-config',
	                    'required' => true,
	                    'values' => array(
		                    array(
			                'id' => 'active_on',
			                'value' => 1,
			                'label' => $this->l('Yes')
		                    ),
		                        array(
			                    'id' => 'active_off',
			                    'value' => 0,
			                    'label' => $this->l('No')
			                    )
	                    ),
	                   ),
					    array(
						'type'     => 'text',
						'label'    => '<span data-toggle="tooltip" title="' . $this->l( 'Get it from your BzPay advisor' ) . '">' . $this->l( 'MONTO PROMOCION DOCE MSI' ) . '<i class="process-icon-help-new help-icon" aria-hidden="true"></i></span>',
						'name'     => 'BZPAY_MSI_12',
						'class'    => 'paylike-config',
						'required' => false
					),
	                   	 array(
	                    'type' => 'switch',
	                    'label'    => '<span data-toggle="tooltip" title="' . $this->l( 'Activar PROMCION' ) . '">' . $this->l( 'PROMO12' ) . '<i class="process-icon-help-new help-icon" aria-hidden="true"></i></span>',
	                    'name'     => 'MSI_PROMO_12',
	                    'is_bool' => true,
	                    'class'    => 'paylike-config',
	                    'required' => true,
	                    'values' => array(
		                    array(
			                'id' => 'active_on',
			                'value' => 1,
			                'label' => $this->l('Yes')
		                    ),
		                        array(
			                    'id' => 'active_off',
			                    'value' => 0,
			                    'label' => $this->l('No')
			                    )
	                    ),
	                   ),
				),
				'submit' => array(
					'title' => $this->l( 'Save' ),
				),

			),
		);


		$helper                           = new HelperForm();
		$helper->show_toolbar             = false;
		$helper->table                    = $this->table;
		$lang                             = new Language( (int) Configuration::get( 'PS_LANG_DEFAULT' ) );
		$helper->default_form_language    = $lang->id;
		$helper->allow_employee_form_lang = Configuration::get( 'PS_BO_ALLOW_EMPLOYEE_FORM_LANG' ) ? Configuration::get( 'PS_BO_ALLOW_EMPLOYEE_FORM_LANG' ) : 0;
		$this->fields_form                = array();

		$helper->identifier    = $this->identifier;
		$helper->submit_action = 'submitBzPay';
		$helper->currentIndex  = $this->context->link->getAdminLink( 'AdminModules', false ) . '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name;
		$helper->token         = Tools::getAdminTokenLite( 'AdminModules' );
		$helper->tpl_vars      = array(
			'fields_value' => $this->getConfigFieldsValues(),
			'languages'    => $this->context->controller->getLanguages(),
			'id_language'  => $this->context->language->id
		);


		$errors = $this->context->controller->errors;
		foreach ( $fields_form['form']['input'] as $key => $field ) {
			if ( array_key_exists( $field['name'], $errors ) ) {
				$fields_form['form']['input'][ $key ]['class'] = ! empty( $fields_form['form']['input'][ $key ]['class'] ) ? $fields_form['form']['input'][ $key ]['class'] . ' has-error' : 'has-error';
			}
		}

		return $helper->generateForm( array( $fields_form ) );
	}
	public function getConfigFieldsValues() {
			if ( ! Configuration::get( 'BZPAY_MERCHANT_ID' ) ) {
				$this->context->controller->errors['BZPAY_MERCHANT_ID'] = $this->l( 'MerchantID required!' );
			}
		
			if ( ! Configuration::get( 'BZPAY_AUTH_KEY' ) ) {
				$this->context->controller->errors['BZPAY_AUTH_KEY'] = $this->l( 'AuthKey required!' );
			}
		if ( ! Configuration::get( 'BZPAY_MSI' ) ) {
				$this->context->controller->errors['BZPAY_MSI'] = $this->l( 'MSI required!' );
			}
			
			if ( ! Configuration::get( 'MSI_PROMO_3' ) ) {
				$this->context->controller->errors['MSI_PROMO_3'] = $this->l( 'MSI amount activation required!' );
			}
			if ( ! Configuration::get( 'MSI_PROMO_6' ) ) {
				$this->context->controller->errors['MSI_PROMO_6'] = $this->l( 'MSI amount activation required!' );
			}
			if ( ! Configuration::get( 'MSI_PROMO_9' ) ) {
				$this->context->controller->errors['MSI_PROMO_9'] = $this->l( 'MSI amount activation required!' );
			}
			if ( ! Configuration::get( 'MSI_PROMO_12' ) ) {
				$this->context->controller->errors['MSI_PROMO_12'] = $this->l( 'MSI amount activation required!' );
			}
			
			

		return array(
			'BZPAY_MERCHANT_ID'=> Configuration::get( 'BZPAY_MERCHANT_ID' ),
			'BZPAY_AUTH_KEY'=> Configuration::get( 'BZPAY_AUTH_KEY' ),
			'BZPAY_MSI'=> Configuration::get( 'BZPAY_MSI' ),
			'BZPAY_MSI_3'=> Configuration::get( 'BZPAY_MSI_3' ),
			'BZPAY_MSI_6'=> Configuration::get( 'BZPAY_MSI_6' ),
			'BZPAY_MSI_9'=> Configuration::get( 'BZPAY_MSI_9' ),
			'BZPAY_MSI_12'=> Configuration::get( 'BZPAY_MSI_12' ),
			'MSI_PROMO_3'=> Configuration::get( 'MSI_PROMO_3' ),
			'MSI_PROMO_6'=> Configuration::get( 'MSI_PROMO_6' ),
			'MSI_PROMO_9'=> Configuration::get( 'MSI_PROMO_9' ),
			'MSI_PROMO_12'=> Configuration::get( 'MSI_PROMO_12' ),
		
		);
	}
	
    public function hookPaymentOptions($params)
    {
		if (!$this->active) {
            return;
        }
		
        if (!$this->checkCurrency($params['cart'])) {
            return;
        }

        $payment_options = [
            
            $this->getExternalPaymentOption($params),
            
            
        ];

        return $payment_options;
    }

    public function checkCurrency($cart)
    {
        $currency_order = new Currency($cart->id_currency);
        $currencies_module = $this->getCurrency($cart->id_currency);

        if (is_array($currencies_module)) {
            foreach ($currencies_module as $currency_module) {
                if ($currency_order->id == $currency_module['id_currency']) {
                    return true;
                }
            }
        }
        return false;
    }
/**
	 * Return the number that should be used to compute cents from the total amount
	 *
	 * @param $currency_iso_code
	 *
	 * @return int|number
	 */
	public function getBzPayCurrencyMultiplier( $currency_iso_code ) {
		$currency = $this->getBzPayCurrency( $currency_iso_code );
		if ( isset( $currency['exponent'] ) ) {
			return pow( 10, $currency['exponent'] );
		} else {
			return pow( 10, 2 );
		}
	}
   /**
	 * Get Paylike amount to pay
	 *
	 * @param float $total Amount due.
	 * @param       $currency_iso_code
	 *
	 * @return float|int
	 */
	public function getBzPayAmount( $total, $currency_iso_code ) {
		$multiplier = $this->getBzPayCurrency( $currency_iso_code );
		$amount     = ceil( $total * $multiplier ); // round to make sure we are always minor units.
		if ( function_exists( 'bcmul' ) ) {
			$amount = ceil( bcmul( $total, $multiplier ) );
		}

		return $amount;
	}

    public function getExternalPaymentOption($params)
    {
			if ( ! Configuration::get( 'BZPAY_MERCHANT_ID' ) ) {
				return false;
			} else {
				$BZPAY_MERCHANT_ID = Configuration::get( 'BZPAY_MERCHANT_ID' );
				
			}
		

		if (  ! Configuration::get( 'BZPAY_AUTH_KEY' ) ) {
			return false;
		}
		 else {
				$BZPAY_AUTH_KEY = Configuration::get( 'BZPAY_AUTH_KEY' );
				
			}
		if (  ! Configuration::get( 'BZPAY_MSI' ) ) {
			return false;
		}
		 else {
				$BZPAY_MSI = Configuration::get( 'BZPAY_MSI' );
				
			}
			
				if (  ! Configuration::get( 'BZPAY_MSI_3' ) ) {
			return false;
		}
		 else {
				$BZPAY_MSI_3 = Configuration::get( 'BZPAY_MSI_3' );
				
			}
			
				if (  ! Configuration::get( 'BZPAY_MSI_6' ) ) {
			return false;
		}
		 else {
				$BZPAY_MSI_6 = Configuration::get( 'BZPAY_MSI_6' );
				
			}
			
				if (  ! Configuration::get( 'BZPAY_MSI_9' ) ) {
			return false;
		}
		 else {
				$BZPAY_MSI_9 = Configuration::get( 'BZPAY_MSI_9' );
				
			}
			
				if (  ! Configuration::get( 'BZPAY_MSI_12' ) ) {
			return false;
		}
		 else {
				$BZPAY_MSI_12 = Configuration::get( 'BZPAY_MSI_12' );
				
			}
			
			///////
				if (  ! Configuration::get( 'MSI_PROMO_3' ) ) {
			return false;
		}
		 else {
				$MSI_PROMO_3 = Configuration::get( 'MSI_PROMO_3' );
				
			}
				if (  ! Configuration::get( 'MSI_PROMO_6' ) ) {
			return false;
		}
		 else {
				$MSI_PROMO_6 = Configuration::get( 'MSI_PROMO_6' );
				
			}
				if (  ! Configuration::get( 'MSI_PROMO_9' ) ) {
			return false;
		}
		 else {
				$MSI_PROMO_9 = Configuration::get( 'MSI_PROMO_9' );
				
			}
				if (  ! Configuration::get( 'MSI_PROMO_12' ) ) {
			return false;
		}
		 else {
				$MSI_PROMO_12 = Configuration::get( 'MSI_PROMO_12' );
				
			}
			
			
		$products       = $params['cart']->getProducts();
		$products_array = array();
		$products_label = array();
		$p              = 0;
		foreach ( $products as $product ) {
			$products_array[]     = array(
				$this->l( 'ID' )       => $product['id_product'],
				$this->l( 'Name' )     => $product['name'],
				$this->l( 'Quantity' ) => $product['cart_quantity']
			);
			$products_label[ $p ] = $product['quantity'] . 'x ' . $product['name'];
			$p ++;
		}
		
		$currency            = new Currency( (int) $params['cart']->id_currency );
		$currency_code       = $currency->iso_code;
		$currency_multiplier = $this->getBzPayCurrencyMultiplier( $currency->iso_code );
		$amount              = $this->getBzPayAmount(  $params['cart']->getOrderTotal(), $currency_multiplier );
		$customer            = new Customer( (int) $params['cart']->id_customer );
		$name                = $customer->firstname . ' ' . $customer->lastname;
		$email               = $customer->email;
		$customer_address    = new Address( (int) ( $params['cart']->id_address_delivery ) );
		$telephone           = ! empty( $customer_address->phone ) ? $customer_address->phone : ( ! empty( $customer_address->phone_mobile ) ? $customer_address->phone_mobile : '' );
		$address             = $customer_address->address1 . ', ' . $customer_address->address2 . ', ' . $customer_address->city . ', ' . $customer_address->country . ' - ' . $customer_address->postcode;
		$ip                  = Tools::getRemoteAddr();
		$locale              = $this->context->language->iso_code;
		$platform_version    = _PS_VERSION_;
		$ecommerce           = 'prestashop';
		$module_version      = $this->version;
		$countrystate = $this->usstate($customer_address->id_state);


		setcookie("address1", $customer_address->address1, time()+3600);
		setcookie("state", $countrystate, time()+3600);
		setcookie("address2", $customer_address->address2, time()+3600);
		setcookie("city", $customer_address->city, time()+3600);
		setcookie("country", $customer_address->country, time()+3600);
		setcookie("postcode", $customer_address->postcode, time()+3600);
		setcookie("phone", ! empty( $customer_address->phone ) ? $customer_address->phone : ( ! empty( $customer_address->phone_mobile ) ? $customer_address->phone_mobile : '' ), time()+3600);
		


		
        $this->context->smarty->assign( array(
			'BZPAY_MERCHANT_ID'              => $BZPAY_MERCHANT_ID,
			'BZPAY_AUTH_KEY'                 => $BZPAY_AUTH_KEY,
			'BZPAY_MSI'                      => $BZPAY_MSI,
			'BZPAY_MSI_3'                    => $BZPAY_MSI_3,
			'BZPAY_MSI_6'                    => $BZPAY_MSI_6,
			'BZPAY_MSI_9'                    => $BZPAY_MSI_9,
			'BZPAY_MSI_12'                   => $BZPAY_MSI_12,
			'MSI_PROMO_3'                    => $MSI_PROMO_3,
			'MSI_PROMO_6'                    => $MSI_PROMO_6,
			'MSI_PROMO_9'                    => $MSI_PROMO_9,
			'MSI_PROMO_12'                   => $MSI_PROMO_12,
			'PS_SSL_ENABLED'                 => ( Configuration::get( 'PS_SSL_ENABLED' ) ? 'https' : 'http' ),
			'http_host'                      => Tools::getHttpHost(),
			'shop_name'                      => $this->context->shop->name,
			'currency_code'                  => $currency_code,
			'amount'                         => $amount,
			'id_cart'                        => Tools::jsonEncode( $params['cart']->id ),
			'products'                       => Tools::jsonEncode( $products_array ),
			'name'                           => $name,
			'email'                          => $email,
			'telephone'                      => $telephone,
			'address'                        => $address,
			'ip'                             => $ip,
			'locale'                         => $locale,
			'platform_version'               => $platform_version,
			'ecommerce'                      => $ecommerce,
			'module_version'                 => $module_version,
			//'redirect_url'                   => $redirect_url,
			//'qry_str'                        => ( Configuration::get( 'PS_REWRITING_SETTINGS' ) ? '?' : '&' ),
			'base_uri'                       => __PS_BASE_URI__,
			'this_path_bzpay'              => $this->_path
		) );
		
        $externalOption = new PaymentOption();
        $externalOption->setCallToActionText($this->l('BzPay Payment'))
                       ->setAction($this->context->link->getModuleLink($this->name, 'validation', array(), true))
                       ->setAdditionalInformation($this->context->smarty->fetch('module:bzpaypayment/views/templates/front/payment_infos.tpl'))
					   //->setAdditionalInformation( $this->display( __FILE__, 'views/templates/front/payment.tpl' ) )
                       ->setLogo(Media::getMediaPath(_PS_MODULE_DIR_.$this->name.'/payment.jpg'));

        return $externalOption;
    }
  function usstate($id){
		
		$stateCode=array (
                          "1"=>"AA",
                          "2"=>"AE",
                          "3"=>"AP",
                          "4"=>"AL",
                          "5"=>"AK",
                          "6"=>"AZ",
                          "7"=>"AR",
                          "8"=>"CA",
                          "9"=>"CO",
                          "10"=>"CT",
                          "11"=>"DE",
                          "12"=>"FL",
                          "13"=>"GA",
                          "14"=>"HI",
                          "15"=>"ID",
                          "16"=>"IL",
                          "17"=>"IN",
                          "18"=>"IA",
                          "19"=>"KS",
                          "20"=>"KY",
                          "21"=>"LA",
                          "22"=>"ME",
                          "23"=>"MD",
                          "24"=>"MA",
                          "25"=>"MI",
                          "26"=>"MN",
                          "27"=>"MS",
                          "28"=>"MO",
                          "29"=>"MT",
                          "30"=>"NE",
                          "31"=>"NV",
                          "32"=>"NH",
                          "33"=>"NJ",
                          "34"=>"NM",
                          "35"=>"NY",
                          "36"=>"NC",
                          "37"=>"ND",
                          "38"=>"OH",
                          "39"=>"OK",
                          "40"=>"OR",
                          "41"=>"PA",
                          "42"=>"RI",
                          "43"=>"SC",
                          "44"=>"SD",
                          "45"=>"TN",
                          "46"=>"TX",
                          "47"=>"UT",
                          "48"=>"VT",
                          "49"=>"VA",
                          "50"=>"WA",
                          "51"=>"WV",
                          "52"=>"WI",
                          "53"=>"WY",
                          "54"=>"PR",
                          "55"=>"VI",
                          "56"=>"DC"
    );
        if( !$stateCode[$id]){ 
		return $stateCode;
		}else{ 
		return $stateCode[$id];
		}
	 
	 }
	public function hookpaymentReturn( $params ) {
		if (!$this->active)
            return;
		//print_r($params['order']);
        $state = $params['order']->getCurrentState();
        if ($state == Configuration::get('PS_OS_PAYMENT') || $state == Configuration::get('PS_OS_OUTOFSTOCK'))
        {
            $id_order = $params['order']->id;
            $history = new OrderHistory();
            $history->id_order = (int)$id_order;
            $history->changeIdOrderState((int)Configuration::get('PS_OS_PAYMENT'), $history->id_order);
            $history->addWithemail();
            $history->save();
            $this->smarty->assign(array(
                'total_to_pay' => Tools::displayPrice($params['total_to_pay'], $params['currencyObj'], false),
                'status' => 'ok',
                'id_order' => $params['order']->id
            ));
            if (isset($params['order']->reference) && !empty($params['order']->reference))
                $this->smarty->assign('reference', $params['order']->reference);
        }
        else
            $this->smarty->assign('status', 'failed');
        	return $this->display( __FILE__, 'views/templates/payment_form.tpl' );
		
	}
   

  /*s  protected function generateForm()
    {
        $months = [];
        for ($i = 1; $i <= 12; $i++) {
            $months[] = sprintf("%02d", $i);
        }

        $years = [];
        for ($i = 0; $i <= 10; $i++) {
            $years[] = date('Y', strtotime('+'.$i.' years'));
        }

        $this->context->smarty->assign([
            'action' => $this->context->link->getModuleLink($this->name, 'validation', array(), true),
            'months' => $months,
            'years' => $years,
        ]);

        return $this->context->smarty->fetch('module:bzpaypayment/views/templates/front/payment_form.tpl');
    }
	*/
	public function storeTransactionID( $paylike_id_transaction, $order_id, $total, $captured = 'NO' ) {
		$query = 'INSERT INTO ' . _DB_PREFIX_ . 'bzpay (`bzpay_tid`, `order_id`, `payed_amount`, `payed_at`, `captured`) VALUES ("' . pSQL( $paylike_id_transaction ) . '", "' . pSQL( $order_id ) . '", "' . pSQL( $total ) . '" , NOW(), "' . pSQL( $captured ) . '")';

		return Db::getInstance()->Execute( $query );
	}
	
	public function getBzPayCurrency( $currency_iso_code ) {
		$currencies = array(
			'AED' =>
				array(
					'code'     => 'AED',
					'currency' => 'United Arab Emirates dirham',
					'numeric'  => '784',
					'exponent' => 2,
				),
			'AFN' =>
				array(
					'code'     => 'AFN',
					'currency' => 'Afghan afghani',
					'numeric'  => '971',
					'exponent' => 2,
				),
			'ALL' =>
				array(
					'code'     => 'ALL',
					'currency' => 'Albanian lek',
					'numeric'  => '008',
					'exponent' => 2,
				),
			'AMD' =>
				array(
					'code'     => 'AMD',
					'currency' => 'Armenian dram',
					'numeric'  => '051',
					'exponent' => 2,
				),
			'ANG' =>
				array(
					'code'     => 'ANG',
					'currency' => 'Netherlands Antillean guilder',
					'numeric'  => '532',
					'exponent' => 2,
				),
			'AOA' =>
				array(
					'code'     => 'AOA',
					'currency' => 'Angolan kwanza',
					'numeric'  => '973',
					'exponent' => 2,
				),
			'ARS' =>
				array(
					'code'     => 'ARS',
					'currency' => 'Argentine peso',
					'numeric'  => '032',
					'exponent' => 2,
				),
			'AUD' =>
				array(
					'code'     => 'AUD',
					'currency' => 'Australian dollar',
					'numeric'  => '036',
					'exponent' => 2,
				),
			'AWG' =>
				array(
					'code'     => 'AWG',
					'currency' => 'Aruban florin',
					'numeric'  => '533',
					'exponent' => 2,
				),
			'AZN' =>
				array(
					'code'     => 'AZN',
					'currency' => 'Azerbaijani manat',
					'numeric'  => '944',
					'exponent' => 2,
				),
			'BAM' =>
				array(
					'code'     => 'BAM',
					'currency' => 'Bosnia and Herzegovina convertible mark',
					'numeric'  => '977',
					'exponent' => 2,
				),
			'BBD' =>
				array(
					'code'     => 'BBD',
					'currency' => 'Barbados dollar',
					'numeric'  => '052',
					'exponent' => 2,
				),
			'BDT' =>
				array(
					'code'     => 'BDT',
					'currency' => 'Bangladeshi taka',
					'numeric'  => '050',
					'exponent' => 2,
				),
			'BGN' =>
				array(
					'code'     => 'BGN',
					'currency' => 'Bulgarian lev',
					'numeric'  => '975',
					'exponent' => 2,
				),
			'BHD' =>
				array(
					'code'     => 'BHD',
					'currency' => 'Bahraini dinar',
					'numeric'  => '048',
					'exponent' => 3,
				),
			'BIF' =>
				array(
					'code'     => 'BIF',
					'currency' => 'Burundian franc',
					'numeric'  => '108',
					'exponent' => 0,
				),
			'BMD' =>
				array(
					'code'     => 'BMD',
					'currency' => 'Bermudian dollar',
					'numeric'  => '060',
					'exponent' => 2,
				),
			'BND' =>
				array(
					'code'     => 'BND',
					'currency' => 'Brunei dollar',
					'numeric'  => '096',
					'exponent' => 2,
				),
			'BOB' =>
				array(
					'code'     => 'BOB',
					'currency' => 'Boliviano',
					'numeric'  => '068',
					'exponent' => 2,
				),
			'BRL' =>
				array(
					'code'     => 'BRL',
					'currency' => 'Brazilian real',
					'numeric'  => '986',
					'exponent' => 2,
				),
			'BSD' =>
				array(
					'code'     => 'BSD',
					'currency' => 'Bahamian dollar',
					'numeric'  => '044',
					'exponent' => 2,
				),
			'BTN' =>
				array(
					'code'     => 'BTN',
					'currency' => 'Bhutanese ngultrum',
					'numeric'  => '064',
					'exponent' => 2,
				),
			'BWP' =>
				array(
					'code'     => 'BWP',
					'currency' => 'Botswana pula',
					'numeric'  => '072',
					'exponent' => 2,
				),
			'BYR' =>
				array(
					'code'     => 'BYR',
					'currency' => 'Belarusian ruble',
					'numeric'  => '974',
					'exponent' => 0,
				),
			'BZD' =>
				array(
					'code'     => 'BZD',
					'currency' => 'Belize dollar',
					'numeric'  => '084',
					'exponent' => 2,
				),
			'CAD' =>
				array(
					'code'     => 'CAD',
					'currency' => 'Canadian dollar',
					'numeric'  => '124',
					'exponent' => 2,
				),
			'CDF' =>
				array(
					'code'     => 'CDF',
					'currency' => 'Congolese franc',
					'numeric'  => '976',
					'exponent' => 2,
				),
			'CHF' =>
				array(
					'code'     => 'CHF',
					'currency' => 'Swiss franc',
					'numeric'  => '756',
					'funding'  => true,
					'exponent' => 2,
				),
			'CLP' =>
				array(
					'code'     => 'CLP',
					'currency' => 'Chilean peso',
					'numeric'  => '152',
					'exponent' => 0,
				),
			'CNY' =>
				array(
					'code'     => 'CNY',
					'currency' => 'Chinese yuan',
					'numeric'  => '156',
					'exponent' => 2,
				),
			'COP' =>
				array(
					'code'     => 'COP',
					'currency' => 'Colombian peso',
					'numeric'  => '170',
					'exponent' => 2,
				),
			'CRC' =>
				array(
					'code'     => 'CRC',
					'currency' => 'Costa Rican colon',
					'numeric'  => '188',
					'exponent' => 2,
				),
			'CUP' =>
				array(
					'code'     => 'CUP',
					'currency' => 'Cuban peso',
					'numeric'  => '192',
					'exponent' => 2,
				),
			'CVE' =>
				array(
					'code'     => 'CVE',
					'currency' => 'Cape Verde escudo',
					'numeric'  => '132',
					'exponent' => 2,
				),
			'CZK' =>
				array(
					'code'     => 'CZK',
					'currency' => 'Czech koruna',
					'numeric'  => '203',
					'exponent' => 2,
				),
			'DJF' =>
				array(
					'code'     => 'DJF',
					'currency' => 'Djiboutian franc',
					'numeric'  => '262',
					'exponent' => 0,
				),
			'DKK' =>
				array(
					'code'     => 'DKK',
					'currency' => 'Danish krone',
					'numeric'  => '208',
					'funding'  => true,
					'exponent' => 2,
				),
			'DOP' =>
				array(
					'code'     => 'DOP',
					'currency' => 'Dominican peso',
					'numeric'  => '214',
					'exponent' => 2,
				),
			'DZD' =>
				array(
					'code'     => 'DZD',
					'currency' => 'Algerian dinar',
					'numeric'  => '012',
					'exponent' => 2,
				),
			'EGP' =>
				array(
					'code'     => 'EGP',
					'currency' => 'Egyptian pound',
					'numeric'  => '818',
					'exponent' => 2,
				),
			'ERN' =>
				array(
					'code'     => 'ERN',
					'currency' => 'Eritrean nakfa',
					'numeric'  => '232',
					'exponent' => 2,
				),
			'ETB' =>
				array(
					'code'     => 'ETB',
					'currency' => 'Ethiopian birr',
					'numeric'  => '230',
					'exponent' => 2,
				),
			'EUR' =>
				array(
					'code'     => 'EUR',
					'currency' => 'Euro',
					'numeric'  => '978',
					'funding'  => true,
					'exponent' => 2,
				),
			'FJD' =>
				array(
					'code'     => 'FJD',
					'currency' => 'Fiji dollar',
					'numeric'  => '242',
					'exponent' => 2,
				),
			'FKP' =>
				array(
					'code'     => 'FKP',
					'currency' => 'Falkland Islands pound',
					'numeric'  => '238',
					'exponent' => 2,
				),
			'GBP' =>
				array(
					'code'     => 'GBP',
					'currency' => 'Pound sterling',
					'numeric'  => '826',
					'funding'  => true,
					'exponent' => 2,
				),
			'GEL' =>
				array(
					'code'     => 'GEL',
					'currency' => 'Georgian lari',
					'numeric'  => '981',
					'exponent' => 2,
				),
			'GHS' =>
				array(
					'code'     => 'GHS',
					'currency' => 'Ghanaian cedi',
					'numeric'  => '936',
					'exponent' => 2,
				),
			'GIP' =>
				array(
					'code'     => 'GIP',
					'currency' => 'Gibraltar pound',
					'numeric'  => '292',
					'exponent' => 2,
				),
			'GMD' =>
				array(
					'code'     => 'GMD',
					'currency' => 'Gambian dalasi',
					'numeric'  => '270',
					'exponent' => 2,
				),
			'GNF' =>
				array(
					'code'     => 'GNF',
					'currency' => 'Guinean franc',
					'numeric'  => '324',
					'exponent' => 0,
				),
			'GTQ' =>
				array(
					'code'     => 'GTQ',
					'currency' => 'Guatemalan quetzal',
					'numeric'  => '320',
					'exponent' => 2,
				),
			'GYD' =>
				array(
					'code'     => 'GYD',
					'currency' => 'Guyanese dollar',
					'numeric'  => '328',
					'exponent' => 2,
				),
			'HKD' =>
				array(
					'code'     => 'HKD',
					'currency' => 'Hong Kong dollar',
					'numeric'  => '344',
					'exponent' => 2,
				),
			'HNL' =>
				array(
					'code'     => 'HNL',
					'currency' => 'Honduran lempira',
					'numeric'  => '340',
					'exponent' => 2,
				),
			'HRK' =>
				array(
					'code'     => 'HRK',
					'currency' => 'Croatian kuna',
					'numeric'  => '191',
					'exponent' => 2,
				),
			'HTG' =>
				array(
					'code'     => 'HTG',
					'currency' => 'Haitian gourde',
					'numeric'  => '332',
					'exponent' => 2,
				),
			'HUF' =>
				array(
					'code'     => 'HUF',
					'currency' => 'Hungarian forint',
					'numeric'  => '348',
					'funding'  => true,
					'exponent' => 2,
				),
			'IDR' =>
				array(
					'code'     => 'IDR',
					'currency' => 'Indonesian rupiah',
					'numeric'  => '360',
					'exponent' => 2,
				),
			'ILS' =>
				array(
					'code'     => 'ILS',
					'currency' => 'Israeli new shekel',
					'numeric'  => '376',
					'exponent' => 2,
				),
			'INR' =>
				array(
					'code'     => 'INR',
					'currency' => 'Indian rupee',
					'numeric'  => '356',
					'exponent' => 2,
				),
			'IQD' =>
				array(
					'code'     => 'IQD',
					'currency' => 'Iraqi dinar',
					'numeric'  => '368',
					'exponent' => 3,
				),
			'IRR' =>
				array(
					'code'     => 'IRR',
					'currency' => 'Iranian rial',
					'numeric'  => '364',
					'exponent' => 2,
				),
			'ISK' =>
				array(
					'code'     => 'ISK',
					'currency' => 'Icelandic króna',
					'numeric'  => '352',
					'exponent' => 2,
				),
			'JMD' =>
				array(
					'code'     => 'JMD',
					'currency' => 'Jamaican dollar',
					'numeric'  => '388',
					'exponent' => 2,
				),
			'JOD' =>
				array(
					'code'     => 'JOD',
					'currency' => 'Jordanian dinar',
					'numeric'  => '400',
					'exponent' => 3,
				),
			'JPY' =>
				array(
					'code'     => 'JPY',
					'currency' => 'Japanese yen',
					'numeric'  => '392',
					'exponent' => 0,
				),
			'KES' =>
				array(
					'code'     => 'KES',
					'currency' => 'Kenyan shilling',
					'numeric'  => '404',
					'exponent' => 2,
				),
			'KGS' =>
				array(
					'code'     => 'KGS',
					'currency' => 'Kyrgyzstani som',
					'numeric'  => '417',
					'exponent' => 2,
				),
			'KHR' =>
				array(
					'code'     => 'KHR',
					'currency' => 'Cambodian riel',
					'numeric'  => '116',
					'exponent' => 2,
				),
			'KMF' =>
				array(
					'code'     => 'KMF',
					'currency' => 'Comoro franc',
					'numeric'  => '174',
					'exponent' => 0,
				),
			'KPW' =>
				array(
					'code'     => 'KPW',
					'currency' => 'North Korean won',
					'numeric'  => '408',
					'exponent' => 2,
				),
			'KRW' =>
				array(
					'code'     => 'KRW',
					'currency' => 'South Korean won',
					'numeric'  => '410',
					'exponent' => 0,
				),
			'KWD' =>
				array(
					'code'     => 'KWD',
					'currency' => 'Kuwaiti dinar',
					'numeric'  => '414',
					'exponent' => 3,
				),
			'KYD' =>
				array(
					'code'     => 'KYD',
					'currency' => 'Cayman Islands dollar',
					'numeric'  => '136',
					'exponent' => 2,
				),
			'KZT' =>
				array(
					'code'     => 'KZT',
					'currency' => 'Kazakhstani tenge',
					'numeric'  => '398',
					'exponent' => 2,
				),
			'LAK' =>
				array(
					'code'     => 'LAK',
					'currency' => 'Lao kip',
					'numeric'  => '418',
					'exponent' => 2,
				),
			'LBP' =>
				array(
					'code'     => 'LBP',
					'currency' => 'Lebanese pound',
					'numeric'  => '422',
					'exponent' => 2,
				),
			'LKR' =>
				array(
					'code'     => 'LKR',
					'currency' => 'Sri Lankan rupee',
					'numeric'  => '144',
					'exponent' => 2,
				),
			'LRD' =>
				array(
					'code'     => 'LRD',
					'currency' => 'Liberian dollar',
					'numeric'  => '430',
					'exponent' => 2,
				),
			'LSL' =>
				array(
					'code'     => 'LSL',
					'currency' => 'Lesotho loti',
					'numeric'  => '426',
					'exponent' => 2,
				),
			'MAD' =>
				array(
					'code'     => 'MAD',
					'currency' => 'Moroccan dirham',
					'numeric'  => '504',
					'exponent' => 2,
				),
			'MDL' =>
				array(
					'code'     => 'MDL',
					'currency' => 'Moldovan leu',
					'numeric'  => '498',
					'exponent' => 2,
				),
			'MGA' =>
				array(
					'code'     => 'MGA',
					'currency' => 'Malagasy ariary',
					'numeric'  => '969',
					'exponent' => 2,
				),
			'MKD' =>
				array(
					'code'     => 'MKD',
					'currency' => 'Macedonian denar',
					'numeric'  => '807',
					'exponent' => 2,
				),
			'MMK' =>
				array(
					'code'     => 'MMK',
					'currency' => 'Myanmar kyat',
					'numeric'  => '104',
					'exponent' => 2,
				),
			'MNT' =>
				array(
					'code'     => 'MNT',
					'currency' => 'Mongolian tögrög',
					'numeric'  => '496',
					'exponent' => 2,
				),
			'MOP' =>
				array(
					'code'     => 'MOP',
					'currency' => 'Macanese pataca',
					'numeric'  => '446',
					'exponent' => 2,
				),
			'MRU' =>
				array(
					'code'     => 'MRU',
					'currency' => 'Mauritanian ouguiya',
					'numeric'  => '929',
					'exponent' => 2,
				),
			'MUR' =>
				array(
					'code'     => 'MUR',
					'currency' => 'Mauritian rupee',
					'numeric'  => '480',
					'exponent' => 2,
				),
			'MVR' =>
				array(
					'code'     => 'MVR',
					'currency' => 'Maldivian rufiyaa',
					'numeric'  => '462',
					'exponent' => 2,
				),
			'MWK' =>
				array(
					'code'     => 'MWK',
					'currency' => 'Malawian kwacha',
					'numeric'  => '454',
					'exponent' => 2,
				),
			'MXN' =>
				array(
					'code'     => 'MXN',
					'currency' => 'Mexican peso',
					'numeric'  => '484',
					'exponent' => 2,
				),
			'MYR' =>
				array(
					'code'     => 'MYR',
					'currency' => 'Malaysian ringgit',
					'numeric'  => '458',
					'exponent' => 2,
				),
			'MZN' =>
				array(
					'code'     => 'MZN',
					'currency' => 'Mozambican metical',
					'numeric'  => '943',
					'exponent' => 2,
				),
			'NAD' =>
				array(
					'code'     => 'NAD',
					'currency' => 'Namibian dollar',
					'numeric'  => '516',
					'exponent' => 2,
				),
			'NGN' =>
				array(
					'code'     => 'NGN',
					'currency' => 'Nigerian naira',
					'numeric'  => '566',
					'exponent' => 2,
				),
			'NIO' =>
				array(
					'code'     => 'NIO',
					'currency' => 'Nicaraguan córdoba',
					'numeric'  => '558',
					'exponent' => 2,
				),
			'NOK' =>
				array(
					'code'     => 'NOK',
					'currency' => 'Norwegian krone',
					'numeric'  => '578',
					'funding'  => true,
					'exponent' => 2,
				),
			'NPR' =>
				array(
					'code'     => 'NPR',
					'currency' => 'Nepalese rupee',
					'numeric'  => '524',
					'exponent' => 2,
				),
			'NZD' =>
				array(
					'code'     => 'NZD',
					'currency' => 'New Zealand dollar',
					'numeric'  => '554',
					'exponent' => 2,
				),
			'OMR' =>
				array(
					'code'     => 'OMR',
					'currency' => 'Omani rial',
					'numeric'  => '512',
					'exponent' => 3,
				),
			'PAB' =>
				array(
					'code'     => 'PAB',
					'currency' => 'Panamanian balboa',
					'numeric'  => '590',
					'exponent' => 2,
				),
			'PEN' =>
				array(
					'code'     => 'PEN',
					'currency' => 'Peruvian Sol',
					'numeric'  => '604',
					'exponent' => 2,
				),
			'PGK' =>
				array(
					'code'     => 'PGK',
					'currency' => 'Papua New Guinean kina',
					'numeric'  => '598',
					'exponent' => 2,
				),
			'PHP' =>
				array(
					'code'     => 'PHP',
					'currency' => 'Philippine peso',
					'numeric'  => '608',
					'exponent' => 2,
				),
			'PKR' =>
				array(
					'code'     => 'PKR',
					'currency' => 'Pakistani rupee',
					'numeric'  => '586',
					'exponent' => 2,
				),
			'PLN' =>
				array(
					'code'     => 'PLN',
					'currency' => 'Polish złoty',
					'numeric'  => '985',
					'funding'  => true,
					'exponent' => 2,
				),
			'PYG' =>
				array(
					'code'     => 'PYG',
					'currency' => 'Paraguayan guaraní',
					'numeric'  => '600',
					'exponent' => 0,
				),
			'QAR' =>
				array(
					'code'     => 'QAR',
					'currency' => 'Qatari riyal',
					'numeric'  => '634',
					'exponent' => 2,
				),
			'RON' =>
				array(
					'code'     => 'RON',
					'currency' => 'Romanian leu',
					'numeric'  => '946',
					'funding'  => true,
					'exponent' => 2,
				),
			'RSD' =>
				array(
					'code'     => 'RSD',
					'currency' => 'Serbian dinar',
					'numeric'  => '941',
					'exponent' => 2,
				),
			'RUB' =>
				array(
					'code'     => 'RUB',
					'currency' => 'Russian ruble',
					'numeric'  => '643',
					'exponent' => 2,
				),
			'RWF' =>
				array(
					'code'     => 'RWF',
					'currency' => 'Rwandan franc',
					'numeric'  => '646',
					'exponent' => 0,
				),
			'SAR' =>
				array(
					'code'     => 'SAR',
					'currency' => 'Saudi riyal',
					'numeric'  => '682',
					'exponent' => 2,
				),
			'SBD' =>
				array(
					'code'     => 'SBD',
					'currency' => 'Solomon Islands dollar',
					'numeric'  => '090',
					'exponent' => 2,
				),
			'SCR' =>
				array(
					'code'     => 'SCR',
					'currency' => 'Seychelles rupee',
					'numeric'  => '690',
					'exponent' => 2,
				),
			'SDG' =>
				array(
					'code'     => 'SDG',
					'currency' => 'Sudanese pound',
					'numeric'  => '938',
					'exponent' => 2,
				),
			'SEK' =>
				array(
					'code'     => 'SEK',
					'currency' => 'Swedish krona',
					'numeric'  => '752',
					'funding'  => true,
					'exponent' => 2,
				),
			'SGD' =>
				array(
					'code'     => 'SGD',
					'currency' => 'Singapore dollar',
					'numeric'  => '702',
					'exponent' => 2,
				),
			'SHP' =>
				array(
					'code'     => 'SHP',
					'currency' => 'Saint Helena pound',
					'numeric'  => '654',
					'exponent' => 2,
				),
			'SLL' =>
				array(
					'code'     => 'SLL',
					'currency' => 'Sierra Leonean leone',
					'numeric'  => '694',
					'exponent' => 2,
				),
			'SOS' =>
				array(
					'code'     => 'SOS',
					'currency' => 'Somali shilling',
					'numeric'  => '706',
					'exponent' => 2,
				),
			'SRD' =>
				array(
					'code'     => 'SRD',
					'currency' => 'Surinamese dollar',
					'numeric'  => '968',
					'exponent' => 2,
				),
			'STN' =>
				array(
					'code'     => 'STN',
					'currency' => 'São Tomé and Príncipe dobra',
					'numeric'  => '930',
					'exponent' => 2,
				),
			'SYP' =>
				array(
					'code'     => 'SYP',
					'currency' => 'Syrian pound',
					'numeric'  => '760',
					'exponent' => 2,
				),
			'SZL' =>
				array(
					'code'     => 'SZL',
					'currency' => 'Swazi lilangeni',
					'numeric'  => '748',
					'exponent' => 2,
				),
			'THB' =>
				array(
					'code'     => 'THB',
					'currency' => 'Thai baht',
					'numeric'  => '764',
					'exponent' => 2,
				),
			'TJS' =>
				array(
					'code'     => 'TJS',
					'currency' => 'Tajikistani somoni',
					'numeric'  => '972',
					'exponent' => 2,
				),
			'TMT' =>
				array(
					'code'     => 'TMT',
					'currency' => 'Turkmenistani manat',
					'numeric'  => '934',
					'exponent' => 2,
				),
			'TND' =>
				array(
					'code'     => 'TND',
					'currency' => 'Tunisian dinar',
					'numeric'  => '788',
					'exponent' => 3,
				),
			'TOP' =>
				array(
					'code'     => 'TOP',
					'currency' => 'Tongan paʻanga',
					'numeric'  => '776',
					'exponent' => 2,
				),
			'TRY' =>
				array(
					'code'     => 'TRY',
					'currency' => 'Turkish lira',
					'numeric'  => '949',
					'exponent' => 2,
				),
			'TTD' =>
				array(
					'code'     => 'TTD',
					'currency' => 'Trinidad and Tobago dollar',
					'numeric'  => '780',
					'exponent' => 2,
				),
			'TWD' =>
				array(
					'code'     => 'TWD',
					'currency' => 'New Taiwan dollar',
					'numeric'  => '901',
					'exponent' => 2,
				),
			'TZS' =>
				array(
					'code'     => 'TZS',
					'currency' => 'Tanzanian shilling',
					'numeric'  => '834',
					'exponent' => 2,
				),
			'UAH' =>
				array(
					'code'     => 'UAH',
					'currency' => 'Ukrainian hryvnia',
					'numeric'  => '980',
					'exponent' => 2,
				),
			'UGX' =>
				array(
					'code'     => 'UGX',
					'currency' => 'Ugandan shilling',
					'numeric'  => '800',
					'exponent' => 0,
				),
			'USD' =>
				array(
					'code'     => 'USD',
					'currency' => 'United States dollar',
					'numeric'  => '840',
					'funding'  => true,
					'exponent' => 2,
				),
			'UYU' =>
				array(
					'code'     => 'UYU',
					'currency' => 'Uruguayan peso',
					'numeric'  => '858',
					'exponent' => 2,
				),
			'UZS' =>
				array(
					'code'     => 'UZS',
					'currency' => 'Uzbekistan som',
					'numeric'  => '860',
					'exponent' => 2,
				),
			'VES' =>
				array(
					'code'     => 'VEF',
					'currency' => 'Venezuelan bolívar',
					'numeric'  => '928',
					'exponent' => 2,
				),
			'VND' =>
				array(
					'code'     => 'VND',
					'currency' => 'Vietnamese dong',
					'numeric'  => '704',
					'exponent' => 0,
				),
			'VUV' =>
				array(
					'code'     => 'VUV',
					'currency' => 'Vanuatu vatu',
					'numeric'  => '548',
					'exponent' => 0,
				),
			'WST' =>
				array(
					'code'     => 'WST',
					'currency' => 'Samoan tala',
					'numeric'  => '882',
					'exponent' => 2,
				),
			'XAF' =>
				array(
					'code'     => 'XAF',
					'currency' => 'CFA franc BEAC',
					'numeric'  => '950',
					'exponent' => 0,
				),
			'XCD' =>
				array(
					'code'     => 'XCD',
					'currency' => 'East Caribbean dollar',
					'numeric'  => '951',
					'exponent' => 2,
				),
			'XOF' =>
				array(
					'code'     => 'XOF',
					'currency' => 'CFA franc BCEAO',
					'numeric'  => '952',
					'exponent' => 0,
				),
			'XPF' =>
				array(
					'code'     => 'XPF',
					'currency' => 'CFP franc',
					'numeric'  => '953',
					'exponent' => 0,
				),
			'YER' =>
				array(
					'code'     => 'YER',
					'currency' => 'Yemeni rial',
					'numeric'  => '886',
					'exponent' => 2,
				),
			'ZAR' =>
				array(
					'code'     => 'ZAR',
					'currency' => 'South African rand',
					'numeric'  => '710',
					'exponent' => 2,
				),
			'ZMK' =>
				array(
					'code'     => 'ZMK',
					'currency' => 'Zambian kwacha',
					'numeric'  => '894',
					'exponent' => 2,
				),
			'ZWL' =>
				array(
					'code'     => 'ZWL',
					'currency' => 'Zimbabwean dollar',
					'numeric'  => '716',
					'exponent' => 2,
				),
		);
		if ( isset( $currencies[ $currency_iso_code ] ) ) {
			return $currencies[ $currency_iso_code ];
		} else {
			return null;
		}
	}
}
